# ------------------------------------------------------------ #
# ------------------- Rutina para calcular ------------------- #
# ----------------- el origen de un episodio ----------------- #
# ------------------------------------------------------------ #

# ------------------------------------------------------------
# Obtencin de una muestra truncada a partir de una estimacin no paramtrica de la densidad

# x			Datos de los que se desea extraer la muestra
# n			Nmero de datos que se van a simular
# a			Valor de truncamiento

rnpt=function(x,n,a){
	val=numeric(n)
	x=x[!is.na(x)]
	lx=length(x)
	h=bw.nrd0(x)
	
	i=1
	while(i<=n){
		l=sample(1:lx,1)
		aux=x[l]+h*rnorm(1)
		if (aux>a){
			val[i]=aux
			i=i+1
		}
	}
	
	return(val)
}


# ------------------------------------------------------------
# Clculo de una muestra de fondo

# MODELO HISTRICO
# his_f			Datos histricos de fondo en la estacin
# vmin			Valor mnimo prefijado para la medicin de la variable
# ndatos		Nmero de datos que se van a simular

mod_historico=function(his_f,vmin,ndatos){

	prop_e=length(his_f[his_f==vmin])/length(his_f[!is.na(his_f)])
	runi=runif(ndatos)
	mues_f=c(rep(vmin,length(runi[runi<=prop_e])),rnpt(x=his_f[his_f!=vmin],n=length(runi[runi>prop_e]),a=vmin))
	
	return(mues_f)
}


# ------------------------------------------------------------
# Clculo de la probabilidad utilizando ratios

# MODELO DE RATIOS
# inmi_e		Datos de inmisin de la estacin
# emi_ct		Datos de emisin de la Central Trmica
# mues_f		Muestra de fondo
# mues_cc		Muestra del Ciclo Combinado
# nmin			Valor mnimo prefijado para la medicin de NOx
# ndatos		Nmero de datos que se van a simular

mod_ratio=function(inmi_e,emi_ct,mues_f,mues_cc,nmin,ndatos){

	# Ratios de inmisin
	inmi_e[inmi_e<=0]=NA
	mues_f[mues_f<nmin]=nmin
	mues_cc[mues_cc<0]=0
	mues_sin_fcc=inmi_e[2]-mues_f-mues_cc
	mues_sin_fcc[mues_sin_fcc<nmin]=nmin
	ratio_inmi=array(0,dim=c(ndatos,2))
	ratio_inmi[,1]=inmi_e[1]/mues_sin_fcc
	ratio_inmi[,2]=mues_sin_fcc/inmi_e[1]
	
	# Ratios de emisin
	emi_ct[emi_ct<=0]=NA
	sum_ct=apply(emi_ct,c(1,2),sum,na.rm=T)
	sum_ct[sum_ct<=0]=NA
	ratio_emi_ct=array(0,dim=c(ndatos,2))
	ratio_emi_ct[,1]=sum_ct[,1]/sum_ct[,2]
	ratio_emi_ct[,2]=sum_ct[,2]/sum_ct[,1]
	
	# Estimacin y comparacin de las densidades
	prob_ct=rep(-1,2)
	auxp=seq(0,1,l=1000)
	for (i in 1:2){
		limx=range(c(ratio_inmi[,i],ratio_emi_ct[,i]),na.rm=T)
		# Se necesitan al menos dos datos de inmisin y dos de emisin
		if(length(ratio_inmi[!is.na(ratio_inmi[,i]),i])>1 & length(ratio_emi_ct[!is.na(ratio_emi_ct[,i]),i])>1){
			dens_inmi=density(ratio_inmi[,i],from=limx[1],to=limx[2],na.rm=T)$y
			dens_emi_ct=density(ratio_emi_ct[,i],from=limx[1],to=limx[2],na.rm=T)$y
			aux=dens_inmi
			aux=apply((aux-outer(dens_emi_ct,auxp,"*"))^2,2,sum)
			prob_ct[i]=auxp[which.min(aux)]*100
		}
	}
	
	return(list(prob_ct=prob_ct))
}


# ------------------------------------------------------------
# Clculo de la incidencia del viento

# MODELO DE VIENTO
# dis_e			Distancia del Ciclo Combinado a la estacin
# dir_e			Direccin (ngulo respecto al Norte) de la recta que une el Ciclo Combinado con la estacin
# dir_v			Direccin (ngulo respecto al Norte) del viento en cada instante
# vel_v			Velocidad del viento en cada instante
# c1,c2			Constantes para el clculo de los grados de libertad
# ndatos		Nmero de datos que se van a simular

mod_viento=function(dis_e,dir_e,dir_v,vel_v,c1,c2,ndatos){
	dir_v[dir_v<0]=NA
	vel_v[vel_v<0]=NA
	ninst=length(dir_v)
	nval=length(dir_v[!is.na(dir_v)])
	prob_v=-1
	
	if (nval>0){
		proy_v=rep(0,ninst)
		glib_e=rep(0,ninst)
		mues_e=array(0,dim=c(ndatos,ninst))
		
		for (i in 1:ninst){
			if (!is.na(dir_v[i]) & !is.na(vel_v[i])){
				proy_v[i]=vel_v[i]*cos((dir_v[i]-dir_e)*pi/180)
				if (proy_v[i]>0 | vel_v[i]==0){
					glib_e[i]=c1*log(exp(c2/c1)+proy_v[i])
					mues_e[,i]=rchisq(ndatos,df=glib_e[i])
				}
			}
		}
		
		Fn=ecdf(mues_e)
		prob_v=(1-abs(2*Fn(dis_e)-1))*100
	}
	
	return(prob_v)
}


# ------------------------------------------------------------
# Clculo de la probabilidad utilizando ratios y proporciones

# MODELO MIXTO (Etapa 1)
# inmi_e		Datos de inmisin de la estacin
# emi_ct		Datos de emisin de la Central Trmica
# mues_f		Muestra de fondo
# nmin			Valor mnimo prefijado para la medicin de NOx

mod_mixto_1=function(inmi_e,emi_ct,mues_f,nmin){
	inmi_e[inmi_e<=0]=NA
	mues_f[mues_f<nmin]=nmin
	
	# Ratios de emisin
	emi_ct[emi_ct<=0]=NA
	sum_ct=apply(emi_ct,c(1,2),sum,na.rm=T)
	sum_ct[sum_ct<=0]=NA
	ratio_emi_ct=sum_ct[,1]/sum_ct[,2]
	
	# Muestra de la Central Trmica
	mues_ct=inmi_e[1]/ratio_emi_ct
	
	# Muestra sin fondo y Central Trmica
	mues_sin_fct=inmi_e[2]-mues_f-mues_ct
	mues_sin_fct[mues_sin_fct<0]=0
	
	return(list(mues_ct=mues_ct,mues_sin_fct=mues_sin_fct))
}

# MODELO MIXTO (Etapa 2)
# inmi_e		Datos de inmisin de la estacin
# mues_ct		Muestra de la Central Trmica
# mues_ccof		Muestra conjunta del Ciclo Combinado y otros posibles focos
# prob_v		Probabilidad de incidencia del viento

mod_mixto_2=function(inmi_e,mues_ct,mues_ccof,prob_v){
	inmi_e[inmi_e<=0]=NA
	
	prob_ct=rep(-1,2)
	prob_cc=rep(-1,2)
	mues_cc=prob_v*mues_ccof/100
	prob_ct[2]=median(mues_ct/inmi_e[2],na.rm=T)*100
	prob_cc[2]=median(mues_cc/inmi_e[2],na.rm=T)*100
	
	return(list(mues_cc=mues_cc,prob_ct=prob_ct,prob_cc=prob_cc))
}


# ------------------------------------------------------------
# Clculo del origen de un episodio

# RUTINA PRINCIPAL
# vinmi_e		Datos de inmisin de cada estacin
# vemi_ct		Datos de emisin de la Central Trmica
# vemi_ct		Datos de emisin del Ciclo Combinado
# vhis_f		Datos histricos de fondo en cada estacin
# vnom_e		Nombres de las estaciones
# vdis_e		Distancia del Ciclo Combinado a las estaciones
# vdir_e		Direccin (ngulo respecto al Norte) de la recta que une cada estacin con el Ciclo Combinado
# vdir_v		Direccin (ngulo respecto al Norte) del viento en cada instante
# vvel_v		Velocidad del viento en cada instante
# c1,c2			Constantes para el clculo de los grados de libertad
# nmin			Valor mnimo prefijado para la medicin de NOx
# ndatos		Nmero de datos que se van a simular
# s0			Valor mnimo prefijado para el clculo de SO2
# n0			Valor mnimo prefijado para el clculo de NOx

origen=function(vinmi_e,vemi_ct,vemi_cc,vhis_f,vnom_e,vdis_e,vdir_e,vdir_v,vvel_v,c1=3.6,c2=2,nmin=2,ndatos=120,s0=10,n0=10){
	nest=ncol(vinmi_e)
	vinmi_e[vinmi_e<=0]=NA
	vemi_ct[vemi_ct<0]=NA
	vemi_cc[vemi_cc<0]=NA
	vdir_v[vdir_v<0]=NA
	vvel_v[vvel_v<0]=NA
	prob=array(-1,dim=c(4,nest))
	dimnames(prob)=list(c("SO2_CT","NOx_CT","SO2_CC","NOx_CC"),vnom_e)
	prob[3,]=-1
	
	sum_cc=apply(vemi_cc,c(1,2),sum,na.rm=T)
	funciona_cc=ifelse(max(sum_cc[,2])>10,TRUE,FALSE)
	
	for (i in 1:nest){
		if (!is.na(vinmi_e[2,i])){
			vmues_f=mod_historico(vhis_f[,i],nmin,ndatos)
			
			if (!is.na(vinmi_e[1,i]) & vinmi_e[1,i]>s0){
				if (funciona_cc){
					aux=mod_mixto_1(vinmi_e[,i],vemi_ct,vmues_f,nmin)
					vmues_ct=aux$mues_ct
					vmues_ccof=aux$mues_sin_fct
					vprob_v=mod_viento(vdis_e[i],vdir_e[i],vdir_v[,i],vvel_v[,i],c1,c2,ndatos)
					if (vprob_v>=0){
						aux=mod_mixto_2(vinmi_e[,i],vmues_ct,vmues_ccof,vprob_v)
						vmues_cc=aux$mues_cc
						prob[4,i]=aux$prob_cc[2]
						prob[1:2,i]=mod_ratio(vinmi_e[,i],vemi_ct,vmues_f,vmues_cc,nmin,ndatos)$prob_ct
					}
				}
				else{
					prob[4,i]=-1
					vmues_cc=rep(0,ndatos)
					prob[1:2,i]=mod_ratio(vinmi_e[,i],vemi_ct,vmues_f,vmues_cc,nmin,ndatos)$prob_ct
				}
			}
			else{
				if (funciona_cc){
					if (vinmi_e[2,i]>n0){
						vmues_ct=rep(0,ndatos)
						vmues_ccof=vinmi_e[2,i]-vmues_f
						vmues_ccof[vmues_ccof<0]=0
						vprob_v=mod_viento(vdis_e[i],vdir_e[i],vdir_v[,i],vvel_v[,i],c1,c2,ndatos)
						if (vprob_v>=0){
							prob[4,i]=mod_mixto_2(vinmi_e[,i],vmues_ct,vmues_ccof,vprob_v)$prob_cc[2]
						}
					}
				}
				else{
					prob[4,i]=-1
				}
			}
		}
	}
	
	prob[is.na(prob)]=-1
	return(list(prob=prob))
}
